<?php
/***************************************************************************
 * $Id: mybb1.class.php 56 2008-06-08 22:35:41Z Nico $
 * Part of Parsimony Import
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

include_once 'basis.class.php';
 
class myBB1Import extends basisImport  {
	
	var $forumID = null;
	var $topicsAdded = array();
	var $lastPostInThread = array();
	
	function myBB1Import ( $directory, $host, $user, $pass, $db, $forumID, $prefix = 'mybb_', $useXAJAX = false, $debug = false ) {
		$this->basisImport( $directory, $host, $user, $pass, $db, $useXAJAX, $debug );
		
		if ( $this->hasAlert() ) {
			return;
		}		
		
		$this->forumID = (int)$forumID;
		$this->prefix = $prefix;
		
		// Check if tables exist
		$this->checkTableExistance( $this->prefix . 'threads' );
		$this->checkTableExistance( $this->prefix . 'posts' );
		$this->checkTableExistance( $this->prefix . 'forums' );
	}
	
	function prepareForDebug () {
		$this->database->sql_truncate ( array ( $this->prefix . 'threads', $this->prefix . 'posts' ) );
	}	
	
	function replaceUmlaut ( $text ) {
		return str_replace ( array ( '&Auml;', '&auml;', '&Ouml;', '&ouml;', '&Uuml;', '&uuml;', '&szlig;' ), array ( 'Ae', 'ae', 'Oe', 'oe', 'Ue', 'ue', 'szlig' ) , $text );
	}
	
	function insertNewTopic ( $title, $posterName, $time ) {
		$title = $this->replaceUmlaut ( $title );
	
		if ( $this->forumID == null ) {
			$this->alert ( 'Please set a forumID before starting the import!' );
		}
		$insertArray = array (	'subject' => $title,
								'uid' => 0,
								'username' => '',
								'dateline' => $time,
								'fid' => $this->forumID,
								//'topic_status' => 0,
								//'topic_type' => 0,
								'firstpost' => 0 );
		// Default values aus dem Merge System
		$insertArray['icon'] = 0;
		$insertArray['totalratings'] = '0';
		$insertArray['notes'] = '';
		$insertArray['visible'] = '1';
		$insertArray['unapprovedposts'] = '0';
		$insertArray['numratings'] = '0';
		$insertArray['attachmentcount'] = '0';
		$insertArray['poll'] = 0;
		
		$this->database->sql_insert ( $this->prefix . 'threads', $insertArray );
		$this->actualTopicID = $this->database->sql_getID();
		$this->topicsAdded[] = $this->actualTopicID;
		return $this->actualTopicID;
	}
	
	function insertPosting ( $topicID, $posterName, $title, $text, $time ) {
		$text = $this->replaceUmlaut ( $text );
		$title = $this->replaceUmlaut ( $title );
        $text = $this->generateBB($text);
		
		$replyTo = isset ( $lastPostInThread[$topicID] ) ? $lastPostInThread[$topicID] : 0;
		
		$insertArray = array (	'tid' => (int)$topicID,
								'fid' => $this->forumID,
								'uid' => 0,
								'username' => $posterName,
								'dateline' => $time,
								'ipaddress' => '127.0.0.1', 
								//'enable_bbcode' => 1,
								'smilieoff' => 'no',
								'includesig' => 'no',
        						'subject' => $title,
        						//'bbcode_uid' => $this->bbCodeUID,
        						'message' => $text,
								'visible' => 1,
								//'post_checksum' => md5 ( $text ),
								'posthash' => '' );		
		$this->database->sql_insert ( $this->prefix . 'posts', $insertArray );
		
        $this->lastPosting = $this->database->sql_getID();
		$lastPostInThread[$topicID] = $this->lastPosting;
        $this->actualTopicID = $topicID;
        return $this->lastPosting;
	}
	
	function finally() {
		// synchronize inserted topics
		$sql = 'SELECT t.tid, count( p.pid ) AS cp, MAX(p.dateline) AS last_post, MIN(p.pid) AS first_post
			FROM ' . $this->prefix . 'threads t LEFT JOIN ' . $this->prefix . 'posts p
			ON t.tid = p.tid WHERE t.fid = ' . $this->forumID . ' GROUP BY t.tid';
		$result = $this->database->sql_query ( $sql );
		while ( $topicToUpdate = $this->database->sql_row ( $result ) ) {
			$updateArray = array (	'replies' => (int)$topicToUpdate['cp'] - 1,
									'firstpost' => (int)$topicToUpdate['first_post'],
									'lastpost' => (int)$topicToUpdate['last_post']);
			$this->database->sql_update ( $this->prefix . 'threads', $updateArray, array ( 'tid' => (int)$topicToUpdate['tid'] ) );
			
		
		}

		// Synchronize first_posting-data
		$sql = 'UPDATE ' . $this->prefix . 'threads t LEFT JOIN ' . $this->prefix . 'posts p ON t.firstpost = p.pid SET t.username = p.username';
		$this->database->sql_query ( $sql );		
		
		// Synchronize last_posting-data
		$sql = 'SELECT username, tid FROM ' . $this->prefix . 'posts ORDER BY dateline DESC';
		$result = $this->database->sql_query ( $sql );
		$alreadySet = array();
		while ( $entry = $this->database->sql_row ( $result ) ) {
			if ( !in_array ( $entry['tid'], $alreadySet ) ) {
				$updateArray = array ( 'lastposter' => $entry['username'] );
				$this->database->sql_update ( $this->prefix . 'threads', $updateArray, array ( 'tid' => (int) $entry['tid'] ) );
				$alreadySet[] = $entry['tid'];
			}
		}
		
		// synchronize forum
		// borrowed from includes/functions_admin.php of phpBB 2.0.23
		$sql = "SELECT COUNT(pid) AS total FROM " . $this->prefix . "posts WHERE fid = $this->forumID";
		$result = $this->database->sql_query($sql);
		
		$total_posts = 0;		
		if ( $row = $this->database->sql_row($result) )
		{
			$total_posts = ($row['total']) ? $row['total'] : 0;
		}
		
		$sql = 'SELECT COUNT(tid) AS total FROM ' . $this->prefix . 'threads WHERE fid = ' . $this->forumID;
		$result = $this->database->sql_query($sql);
		$row = $this->database->sql_row($result);
		$total_topics = $row['total'];
		
		// Data of the latest posting
		$sql = 'SELECT tid FROM ' . $this->prefix . 'threads WHERE fid = ' . $this->forumID . ' ORDER BY lastpost DESC LIMIT 1';
		$result = $this->database->sql_query ( $sql );
		$row = $this->database->sql_row($result);
		$last_post = $row['tid'];		
		
		$sql = 'SELECT username, dateline, tid as last_post FROM ' . $this->prefix . 'posts WHERE tid = ' . $last_post . ' ORDER BY dateline DESC LIMIT 1';
		$result = $this->database->sql_query ( $sql );
		$row = $this->database->sql_row($result);
		$last_poster = $row['username'];
		$last_post_timestamp = $row['dateline'];
		
		$sql = 'SELECT subject FROM ' . $this->prefix . 'posts WHERE tid = ' . $last_post . ' ORDER BY pid ASC LIMIT 1';
		$result = $this->database->sql_query ( $sql );
		$row = $this->database->sql_row($result);
		$last_post_subject = $row['subject'];
		

		// get old forum data to determine difference
		$sql = 'SELECT threads, posts FROM ' . $this->prefix . 'forums WHERE forum_id = ' . $this->forumID;
		$result = $this->database->sql_query ( $sql );
		$oldStats = $this->database->sql_row ( $result );
		$addedPostings = $total_posts - $oldStats['posts'];
		$addedTopics = $total_topics - $oldStats['threads'];
		
		$updateArray = array (	'lastpost' => (int)$last_post_timestamp,
								'lastposter' => $last_poster,
								'lastposteruid' => 0,
								'lastposttid' => (int)$last_post,
								'lastpostsubject' => $last_post_subject,
							 	'posts' => (int)$total_posts,
								'threads' => (int)$total_topics );
		$this->database->sql_update( $this->prefix . 'forums', $updateArray, array ( 'fid' => $this->forumID ) );
	}
	
	function generateBB ( $text ) {
		$text = $this->generateQuote( $text );
		$text = preg_replace ( "@&amp;lt;i&amp;gt;(.*?)&amp;lt;/i&amp;gt;@s", "[i]$1[/i]", $text );
		
		$text = str_replace ( array ( "&amp;lt;", "&amp;gt;" ), array ( "&lt;", "&gt;" ), $text );
		return $text;
	}
} 
?>